/*
 * Copyright 2019 by Kappich Systemberatung Aachen
 * ALL RIGHTS RESERVED.
 */

package de.bsvrz.dav.daf.accessControl.internal;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;

/**
 * Abstrakte Klasse, die gemeinsame Funktionalitäten von {@link OldUserInfo} und {@link
 * ExtendedUserInfo} beinhaltet und, da sie {@link DataLoader} erweitert, auch für das
 * laden der BenutzerParameter (also die referenzierten Berechtigungsklassen) verantwortlich ist.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 0000 $
 */
abstract class AbstractUserInfo extends DataLoader implements UserInfoInternal {

	protected static final String USER_ASPECT_PID = "asp.parameterSoll";

	/** Zählt wie oft ein UserInfo-Objekt referenziert wurde. */
	private int _referenceCounter = 1;

	/**
	 * Initialisiert die BenutzerInfo-Klasse, indem einem DataLoader Aspekt und Attributgruppe übergeben wird um die BenutzerParameter (Berechtigungsklassen) zu
	 * laden.
	 *
	 * @param accessControlManager
	 * @param connection Verbindung zum Datenverteiler
	 * @param userAttributeGroupPid
	 */
	public AbstractUserInfo(final DafAccessControlManager accessControlManager, ClientDavInterface connection, final String userAttributeGroupPid) {
		super(connection, userAttributeGroupPid, USER_ASPECT_PID, accessControlManager.getUpdateLock());
	}

	@Override
	public final void incrementReference() {
		++_referenceCounter;
	}

	@Override
	public final void decrementReference() {
		--_referenceCounter;
	}

	@Override
	public final boolean canBeSafelyDeleted() {
		return _referenceCounter <= 0;
	}

	/**
	 * Wird aufgerufen, wenn sich die BenutzerParameter für den aktuellen Benutzer ändern.
	 *
	 * @param data Datenobjekt mit den Daten der Attributgruppe atg.benutzerParameter für den aktuellen Benutzer.
	 */
	@Override
	protected abstract void update(final Data data);
}
